import Database from "../../../../../Components/common/DAsyncStorage";
import NetInfo from "@react-native-community/netinfo";
import {
  detectOnlyPoseFileNames,
  DOC,
  get2WPoseFileNotDoc,
  get4WPoseFileNames,
  image_pose_blur,
  image_pose_dark,
  image_pose_wrong,
  is2Wflow,
  multiplePics,
} from "react-native-digitcamera/src/common/Constants";
import { measureConnectionSpeed } from "react-native-network-bandwith-speed";
import { poseDetection } from "react-native-digitcamera/src/common/Services/poseDetection";
import {
  UELogEvent,
  UELogMessage,
} from "react-native-digitcamera/src/common/libEventLog/UserExperior/LibUserExperior";
import { chassiesPoseDetetction } from "react-native-digitcamera/src/common/Services/chassisPoseDetection";

// save data in realm
export const offlinePIDataToRealm = async (isPIDone) => {
  try {
    const mediaStatusObjs = await Database.get("PIMediaStatus");
    const vechicleData = await Database.get("PIVechicleData");
    console.log(
      "!is2Wflow(vechicleData?.activityTypeId)  isPIDone",
      !is2Wflow(vechicleData?.activityTypeId),
      isPIDone
    );
    if (!is2Wflow(vechicleData?.activityTypeId) || isPIDone) {
      console.log("In if offlinePIDataToRealm ");
      let offlineImageData = [];
      console.log(
        "CommonFunc  offlinePIDataToRealm() mediaStatusObjs \n\n\n ",
        mediaStatusObjs
      );
      offlineImageData = [];
      mediaStatusObjs?.filter((item) => {
        if (item?.status === "failed") {
          offlineImageData.push(item);
        }
      });

      await Database.set("PIOfflineVechicleData", {
        activityId: vechicleData.activityId,
        registrationNumber: vechicleData.licensePlateNumber,
        quoteNumber: vechicleData.quoteNumber,
        dataTobeUploaded: JSON.stringify(offlineImageData),
        isPICompleted: isPIDone,
        activityTypeId: vechicleData?.activityTypeId,
      });
    } else {
      console.log("In else offlinePIDataToRealm ");
    }
  } catch (error) {
    console.log("***** offlinePIDataToRealm catch ", error);
  }
};

// checking internet
export const checkInternet = async () => {
  const state = await NetInfo.fetch();
  console.log("Connection type", state.type);
  console.log("Is connected?", state.isConnected);
  return state.isConnected;
};

export const checkInternetSpeed = async () => {
  const DataSpeed = await measureConnectionSpeed();
  console.log(
    "%%%%%%%%%%%% DataSpeed %%%%%%",
    JSON.stringify(DataSpeed, null, "  "),
    DataSpeed?.speed * 1024
  );
  var kbps = DataSpeed?.speed * 1024;
  console.log("KBPS   ", kbps, kbps >= 100);
  if (kbps >= 100) {
    return true;
  }
  return false;
};
// make desion for offline/online flow
export const checkFlow = () => {};

export const clearAllRealmData = async () => {
  try {
    await Database.remove("PIOfflineData");
    await Database.remove("PIVechicleData");
    await Database.remove("PIVechicleInstruction");

    await Database.update("PIGonePoseDetection", {
      GoneposeDetectionCount: 0,
    });

    await Database.update("PIPoseDetectionCompleted", {
      poseDetectionCompletedCount: 0,
    });

    await Database.update("PIRetryCount", {
      retryCount: 0,
      index: 0,
      isAddNewImageData: false,
    });

    await Database.update("PIclickedCount", {
      clickedCount: 0,
    });

    await Database.update("PIUploadedCount", {
      uploadedCount: 0,
    });
    await Database.remove("PIMediaStatus");
    await Database.remove("PIVideoCompleteStatus");
  } catch (error) {}
};

export const updateStatusRealm = async (uri, DocumentImageName, newStatus) => {
  var PIVechicleData = await Database.get("PIVechicleData");
  console.log(
    "CommonFunc.js  updateStatusRealm() PIVechicleDat?.activityId ",
    PIVechicleData?.activityId
  );
  var activityId = PIVechicleData?.activityId;
  var quoteNumber = PIVechicleData?.quoteNumber;
  var licensePlateNumber = PIVechicleData?.licensePlateNumber;
  await initStatusRealm(
    uri,
    DocumentImageName,
    activityId,
    quoteNumber,
    licensePlateNumber
  ).then(async () => {
    const uploadObj = await Database.get("PIMediaStatus");
    console.log(
      "*** CommonFunc.js updateStatusRealm before update status uploadObj: " +
        JSON.stringify(uploadObj),
      newStatus
    );

    if (uploadObj != null) {
      var newObj;
      if (uploadObj instanceof Array) {
        newObj = uploadObj.map((obj) =>
          obj.imageName === DocumentImageName
            ? {
                ...obj,
                imageName: DocumentImageName,
                status: newStatus,
                activityId,
              }
            : obj
        );
      } else {
        newObj = [
          {
            ...uploadObj,
            activityId,
            imageName: DocumentImageName,
            status: newStatus,
          },
        ];
      }
      console.log(
        "*** CommonFunc.js updateStatusRealm newObj: " + JSON.stringify(newObj)
      );
      await Database.set("PIMediaStatus", newObj);
      const alog = await Database.get("PIMediaStatus");
      const aoffline = await Database.get("PIOfflineVechicleData");
      console.log(
        "Media Status Obj updateStatusRealm  PIMediaStatus ",
        JSON.stringify(alog, null, "  ")
      );
      console.log(
        "PIOfflineVechicleData updateStatusRealm  PIOfflineVechicleData ",
        JSON.stringify(aoffline, null, "  ")
      );
    }
  });
};

const addMediaStatusObj = (oldStatus, myObj) => {
  console.log(
    "**** addMediaStatusObj ",
    typeof oldStatus,
    typeof myObj,
    JSON.stringify(oldStatus),
    JSON.stringify(myObj)
  );
  console.log("0 " + myObj);
  var newObj;
  if (oldStatus == null || oldStatus == undefined) {
    console.log("oldStatus == null || oldStatus == undefined", myObj);
    return [myObj];
  }
  if (oldStatus instanceof Array) {
    var isImageExist;
    oldStatus.map((imageItemToupdate) => {
      // if already object present
      if (Object.values(imageItemToupdate).includes(myObj.imageName)) {
        return (isImageExist = true);
      }
      console.log(
        "CommonFunctions.js >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>",
        isImageExist
      );
    });
    if (isImageExist) {
      console.log("*******CommonFunc.js Object already in DB");
      newObj = oldStatus.filter(
        (imageItemToupdate) => imageItemToupdate.imageName != myObj.imageName
      ); // removing previous obj
      console.log(
        "In CommonFunc.js removed Previous Object",
        JSON.stringify(newObj, null, "  ")
      );
      newObj = [...newObj, myObj]; // adding new
      console.log(
        "In CommonFunc.js added with new Object",
        JSON.stringify(newObj, null, "  ")
      );
    } else {
      // no previous obj
      // adding new obj
      newObj = [...oldStatus, myObj];
      console.log("CommonFunc.js oldStatus instanceof Array Added", newObj);
    }
  } else {
    console.log("********** **********   In else ");
    if (oldStatus != undefined || oldStatus != null) {
      newObj = [oldStatus, myObj];
      console.log(
        "******* oldStatus != undefined || oldStatus != null",
        newObj
      );
    } else {
      newObj = [myObj];
      console.log("******* newObj = [myObj]", newObj);
    }
  }
  return newObj;
};

const initStatusRealm = async (
  uri,
  DocumentImageName,
  activityId,
  quoteNumber,
  licensePlateNumber
) => {
  const oldStatus = await Database.get("PIMediaStatus");
  const myObj = {
    status: "uploading", // uploading, uploaded, failed
    uri: uri,
    imageName: DocumentImageName,
    activityId: activityId,
    quoteNumber: quoteNumber,
    regNumber: licensePlateNumber,
  };
  const newObj = addMediaStatusObj(oldStatus, myObj);
  console.log(
    ")))))))))))))))))))))))))))))))))))))))))))))",
    JSON.stringify(newObj, null, "  ")
  );
  // don't change set to update as we are adding new Obj.
  await Database.set("PIMediaStatus", newObj);
  const PIMediaStatus = await Database.get("PIMediaStatus");
  console.log("PIMediaStatus initStatusRealm" + JSON.stringify(PIMediaStatus));
};

// POSE DETECTION API CALL
export const callPoseDetection = async (
  uri,
  DocumentImageName,
  poseDetectionSucess,
  poseDetectionFailed
) => {
  var PIVechicleData = await Database.get("PIVechicleData");
  var PIRetryCount = await Database.get("PIRetryCount");
  var activityId = PIVechicleData?.activityId;
  var licensePlateNumber = PIVechicleData?.licensePlateNumber;
  var activityTypeId = PIVechicleData?.activityTypeId;

  let isBlur = true,
    isDark = true;
  let imgPose;
  let poseName;
  let isDetect;
  let message;
  var formdata = new FormData();
  var ext = DocumentImageName == "VDO" ? ".mp4" : ".jpg";
  formdata.append("file", {
    uri: uri,
    name: activityId + "_" + DocumentImageName + ext,
    fileName: activityId + "_" + DocumentImageName,
    type: "image/jpeg",
  });

  formdata.append("policyNumber", 12312);
  formdata.append("activityTypeId", `${activityTypeId}`);
  formdata.append("checkQuality", true);
  formdata.append("source", "DIGITEDGE");

  console.log("Pose Detection started for ", DocumentImageName);

  const poseDetectionData = await poseDetection(formdata);

  isBlur = poseDetectionData?.image_pose_array[0]?.is_blurred;
  isDark = poseDetectionData?.image_pose_array[0]?.is_dark;
  imgPose = poseDetectionData?.image_pose_array[0]?.image_pose;

  poseName = get4WPoseFileNames();

  // if image is type of document like RC/DL
  if (imgPose == DOC) {
    var isIncorrect = false;
    var poseNameNotDoc = get2WPoseFileNotDoc(); // gating Doc
    poseNameNotDoc.forEach((item) => {
      //RCB
      if (item == DocumentImageName) {
        isIncorrect = true;
      }
    });

    if (!isIncorrect) {
      isDetect == true;
    }
  } else {
    // if it is purly images
    poseName.forEach((item) => {
      if (item == imgPose) {
        if (imgPose == DocumentImageName) {
          isDetect = true;
        }
      }
    });
  }

  if (!detectOnlyPoseFileNames().includes(imgPose) && (isBlur || isDark)) {
    isDetect = false;
  }
  console.log("isBlur", isBlur);
  console.log("isDark", isDark);

  if (!isDetect) {
    UELogEvent("is detect is false", {
      isDetect: JSON.stringify(!isDetect),
    });
    var multiCapturePose = true;
    var multiPoseName = "";

    var pose = multiplePics();
    pose.forEach((item) => {
      if (DocumentImageName !== undefined && DocumentImageName.includes(item)) {
        multiCapturePose = true;
        multiPoseName = item;
      }
    });

    if (isBlur && isDark) {
      message = image_pose_blur + " " + image_pose_dark;
    } else {
      if (isDark) message = image_pose_dark;
      if (isBlur) message = image_pose_blur;
      if (!isDark && !isBlur) message = image_pose_wrong;
    }
  }

  if (!isBlur && !isDark && isDetect) {
    //if image is good then excute this block
    // upload image

    poseDetectionSucess();
  } else {
    poseDetectionFailed();
    //create sensor data ==> if image not  uploaded
    UELogMessage("create sensor data ==> if image not  uploaded");
  }
};

//CALL CHASSIS POSE DETECTION
export const callChassisPoseDetection = async (
  uri,
  DocumentImageName,
  chassiesPoseDetectionSucess,
  chassiesPDetectionFailed
) => {
  var PIRetryCount = await Database.get("PIRetryCount");
  var PIVechicleData = await Database.get("PIVechicleData");
  var activityId = PIVechicleData?.activityId;
  var formdata = new FormData();
  var ext = DocumentImageName == "VDO" ? ".mp4" : ".jpg";

  formdata.append("file", {
    uri: uri,
    name: activityId + "_" + DocumentImageName + ext,
    fileName: activityId + "_" + DocumentImageName,
    type: "image/jpeg",
  });
  console.log("888888888888888888", activityId);
  formdata.append("activityId", activityId);

  let ChassisPoseDetectionData;
  try {
    ChassisPoseDetectionData = await chassiesPoseDetetction({
      body: formdata,
    });

    console.log(
      "chassis pose detection   ",
      JSON.stringify(ChassisPoseDetectionData, null, "  ")
    );
  } catch (e) {
    console.log("***PreviewImage, error in chassiesPoseDetection =>", e);
  }

  if (
    ChassisPoseDetectionData?.match_ratio >
    ChassisPoseDetectionData?.threshold_match_ratio
  ) {
    chassiesPoseDetectionSucess();
  } else {
    chassiesPDetectionFailed();
  }
};
